@@ -9,7 +9,7 @@ import logging
 import asyncio
 import uuid
 from functools import wraps
-from typing import Callable, Any, Union
+from typing import Callable, Any, List, Union
 
 from bleak.backends.device import BLEDevice
 from bleak.backends.dotnet.scanner import BleakScannerDotNet
@@ -96,10 +96,10 @@ class BleakClientDotNet(BaseBleakClient):
 
     Args:
         address_or_ble_device (`BLEDevice` or str): The Bluetooth address of the BLE peripheral to connect to or the `BLEDevice` object representing it.
-        use_cached (bool): If set to `True`, then the OS level BLE cache is used for
-                getting services, characteristics and descriptors. Defaults to ``True``.
 
     Keyword Args:
+        use_cached (bool): If set to `True`, then the OS level BLE cache is used for
+                getting services, characteristics and descriptors. Defaults to ``True``.
         timeout (float): Timeout for required ``BleakScanner.find_device_by_address`` call. Defaults to 10.0.
 
     """
@@ -113,8 +113,8 @@ class BleakClientDotNet(BaseBleakClient):
         else:
             self._device_info = None
         self._requester = None
-        self._connect_events: list[asyncio.Event] = []
-        self._disconnect_events: list[asyncio.Event] = []
+        self._connect_events: List[asyncio.Event] = []
+        self._disconnect_events: List[asyncio.Event] = []
         self._connection_status_changed_token: EventRegistrationToken = None
         self._session: GattSession = None
 
@@ -305,12 +305,17 @@ class BleakClientDotNet(BaseBleakClient):
             else self._requester.ConnectionStatus == BluetoothConnectionStatus.Connected
         )
 
+    @property
+    def mtu_size(self) -> int:
+        """Get ATT MTU size for active connection"""
+        return self._session.MaxPduSize
+
     async def pair(self, protection_level=None, **kwargs) -> bool:
         """Attempts to pair with the device.
 
         Keyword Args:
             protection_level:
-                    DevicePairingProtectionLevel
+                    Windows.Devices.Enumeration.DevicePairingProtectionLevel
                         1: None - Pair the device using no levels of protection.
                         2: Encryption - Pair the device using encryption.
                         3: EncryptionAndAuthentication - Pair the device using
@@ -560,7 +565,6 @@ class BleakClientDotNet(BaseBleakClient):
     async def read_gatt_char(
         self,
         char_specifier: Union[BleakGATTCharacteristic, int, str, uuid.UUID],
-        use_cached=False,
         **kwargs,
     ) -> bytearray:
         """Perform read operation on the specified GATT characteristic.
@@ -569,6 +573,8 @@ class BleakClientDotNet(BaseBleakClient):
             char_specifier (BleakGATTCharacteristic, int, str or UUID): The characteristic to read from,
                 specified by either integer handle, UUID or directly by the
                 BleakGATTCharacteristic object representing it.
+
+        Keyword Args:
             use_cached (bool): ``False`` forces Windows to read the value from the
                 device again and not use its own cached value. Defaults to ``False``.
 
@@ -576,6 +582,9 @@ class BleakClientDotNet(BaseBleakClient):
             (bytearray) The read data.
 
         """
+
+        use_cached = kwargs.get("use_cached", False)
+
         if not isinstance(char_specifier, BleakGATTCharacteristic):
             characteristic = self.services.get_characteristic(char_specifier)
         else:
@@ -620,13 +629,13 @@ class BleakClientDotNet(BaseBleakClient):
                 )
         return value
 
-    async def read_gatt_descriptor(
-        self, handle: int, use_cached=False, **kwargs
-    ) -> bytearray:
+    async def read_gatt_descriptor(self, handle: int, **kwargs) -> bytearray:
         """Perform read operation on the specified GATT descriptor.
 
         Args:
             handle (int): The handle of the descriptor to read from.
+
+        Keyword Args:
             use_cached (bool): ``False`` forces Windows to read the value from the
                 device again and not use its own cached value. Defaults to ``False``.
 
@@ -634,6 +643,8 @@ class BleakClientDotNet(BaseBleakClient):
             (bytearray) The read data.
 
         """
+        use_cached = kwargs.get("use_cached", False)
+
         descriptor = self.services.get_descriptor(handle)
         if not descriptor:
             raise BleakError("Descriptor with handle {0} was not found!".format(handle))
@@ -677,7 +688,7 @@ class BleakClientDotNet(BaseBleakClient):
     async def write_gatt_char(
         self,
         char_specifier: Union[BleakGATTCharacteristic, int, str, uuid.UUID],
-        data: bytearray,
+        data: Union[bytes, bytearray, memoryview],
         response: bool = False,
     ) -> None:
         """Perform a write operation of the specified GATT characteristic.
@@ -738,7 +749,9 @@ class BleakClientDotNet(BaseBleakClient):
                     )
                 )
 
-    async def write_gatt_descriptor(self, handle: int, data: bytearray) -> None:
+    async def write_gatt_descriptor(
+        self, handle: int, data: Union[bytes, bytearray, memoryview]
+    ) -> None:
         """Perform a write operation on the specified GATT descriptor.
 
         Args:
