@@ -1,7 +1,15 @@
 import abc
 import asyncio
 import inspect
-from typing import Callable, Dict, List, Optional, Tuple
+from typing import (
+    Awaitable,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+)
+from warnings import warn
 
 from bleak.backends.device import BLEDevice
 
@@ -48,7 +56,15 @@ class AdvertisementData:
         return f"AdvertisementData({', '.join(kwargs)})"
 
 
-AdvertisementDataCallback = Callable[[BLEDevice, AdvertisementData], None]
+AdvertisementDataCallback = Callable[
+    [BLEDevice, AdvertisementData],
+    Optional[Awaitable[None]],
+]
+
+AdvertisementDataFilter = Callable[
+    [BLEDevice, AdvertisementData],
+    bool,
+]
 
 
 class BaseBleakScanner(abc.ABC):
@@ -82,7 +98,7 @@ class BaseBleakScanner(abc.ABC):
         """
         async with cls(**kwargs) as scanner:
             await asyncio.sleep(timeout)
-            devices = await scanner.get_discovered_devices()
+            devices = scanner.discovered_devices
         return devices
 
     def register_detection_callback(
@@ -93,10 +109,12 @@ class BaseBleakScanner(abc.ABC):
         If another callback has already been registered, it will be replaced with ``callback``.
         ``None`` can be used to remove the current callback.
 
-        The ``callback`` is a function that takes two arguments: :class:`BLEDevice` and :class:`AdvertisementData`.
+        The ``callback`` is a function or coroutine that takes two arguments: :class:`BLEDevice`
+        and :class:`AdvertisementData`.
 
         Args:
-            callback: A function or ``None``.
+            callback: A function, coroutine or ``None``.
+
         """
         if callback is not None:
             error_text = "callback must be callable with 2 parameters"
@@ -107,7 +125,15 @@ class BaseBleakScanner(abc.ABC):
             if len(handler_signature.parameters) != 2:
                 raise TypeError(error_text)
 
-        self._callback = callback
+        if inspect.iscoroutinefunction(callback):
+
+            def detection_callback(s, d):
+                asyncio.ensure_future(callback(s, d))
+
+        else:
+            detection_callback = callback
+
+        self._callback = detection_callback
 
     @abc.abstractmethod
     async def start(self):
@@ -129,20 +155,37 @@ class BaseBleakScanner(abc.ABC):
         """
         raise NotImplementedError()
 
-    @abc.abstractmethod
+    @abc.abstractproperty
+    def discovered_devices(self) -> List[BLEDevice]:
+        """Gets the devices registered by the BleakScanner.
+
+        Returns:
+            A list of the devices that the scanner has discovered during the scanning.
+        """
+        raise NotImplementedError()
+
     async def get_discovered_devices(self) -> List[BLEDevice]:
         """Gets the devices registered by the BleakScanner.
 
+        .. deprecated:: 0.11.0
+            This method will be removed in a future version of Bleak. Use the
+            :attr:`.discovered_devices` property instead.
+
         Returns:
             A list of the devices that the scanner has discovered during the scanning.
 
         """
-        raise NotImplementedError()
+        warn(
+            "This method will be removed in a future version of Bleak. Use the `discovered_devices` property instead.",
+            FutureWarning,
+            stacklevel=2,
+        )
+        return self.discovered_devices
 
     @classmethod
     async def find_device_by_address(
         cls, device_identifier: str, timeout: float = 10.0, **kwargs
-    ) -> BLEDevice:
+    ) -> Optional[BLEDevice]:
         """A convenience method for obtaining a ``BLEDevice`` object specified by Bluetooth address or (macOS) UUID address.
 
         Args:
@@ -157,21 +200,35 @@ class BaseBleakScanner(abc.ABC):
 
         """
         device_identifier = device_identifier.lower()
-        stop_scanning_event = asyncio.Event()
+        return await cls.find_device_by_filter(
+            lambda d, ad: d.address.lower() == device_identifier
+        )
 
-        def stop_if_detected(d: BLEDevice, ad: AdvertisementData):
-            if d.address.lower() == device_identifier:
-                stop_scanning_event.set()
+    @classmethod
+    async def find_device_by_filter(
+        cls, filterfunc: AdvertisementDataFilter, timeout: float = 10.0, **kwargs
+    ) -> Optional[BLEDevice]:
+        """A convenience method for obtaining a ``BLEDevice`` object specified by a filter function.
 
-        async with cls(
-            timeout=timeout, detection_callback=stop_if_detected, **kwargs
-        ) as scanner:
+        Args:
+            filterfunc (AdvertisementDataFilter): A function that is called for every BLEDevice found. It should return True only for the wanted device.
+            timeout (float): Optional timeout to wait for detection of specified peripheral before giving up. Defaults to 10.0 seconds.
+
+        Keyword Args:
+            adapter (str): Bluetooth adapter to use for discovery.
+
+        Returns:
+            The ``BLEDevice`` sought or ``None`` if not detected.
+
+        """
+        found_device_queue = asyncio.Queue()
+
+        def apply_filter(d: BLEDevice, ad: AdvertisementData):
+            if filterfunc(d, ad):
+                found_device_queue.put_nowait(d)
+
+        async with cls(detection_callback=apply_filter, **kwargs):
             try:
-                await asyncio.wait_for(stop_scanning_event.wait(), timeout=timeout)
+                return await asyncio.wait_for(found_device_queue.get(), timeout=timeout)
             except asyncio.TimeoutError:
                 return None
-            return next(
-                d
-                for d in await scanner.get_discovered_devices()
-                if d.address.lower() == device_identifier
-            )
