@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from typing import Optional
 
 
 class BleakError(Exception):
@@ -16,18 +17,38 @@ class BleakDotNetTaskError(BleakError):
 class BleakDBusError(BleakError):
     """Specialized exception type for D-Bus errors."""
 
-    def __init__(self, dbus_error: str):
+    def __init__(self, dbus_error: str, error_body: list):
         """
         Args:
             dbus_error (str): The D-Bus error, e.g. ``org.freedesktop.DBus.Error.UnknownObject``.
+            error_body (list): Body of the D-Bus error, sometimes containing error description or details.
         """
-        super().__init__(dbus_error)
+        super().__init__(dbus_error, *error_body)
 
     @property
     def dbus_error(self) -> str:
         """Gets the D-Bus error name, e.g. ``org.freedesktop.DBus.Error.UnknownObject``."""
         return self.args[0]
 
+    @property
+    def dbus_error_details(self) -> Optional[str]:
+        """Gets the optional D-Bus error details, e.g. 'Invalid UUID'."""
+        if len(self.args) > 1:
+            details = self.args[1]
+            # Some error descriptions can be further parsed to be even more helpful
+            if "ATT error: 0x" in details:
+                more_detail = CONTROLLER_ERROR_CODES.get(
+                    int(details.rsplit("x")[1], 16), "Unknown code"
+                )
+                details += f" ({more_detail})"
+            return details
+        return None
+
+    def __str__(self) -> str:
+        name = f"[{self.dbus_error}]"
+        details = self.dbus_error_details
+        return (name + " " + details) if details else name
+
 
 CONTROLLER_ERROR_CODES = {
     0x00: "Success",
